@@ -98,7 +98,7 @@ enum accidentals {
 #define BEAM_FLATFAC	0.6	/* factor to decrease slope of long beams */
 #define BEAM_THRESH	0.06	/* flat beam if slope below this threshold */
 #define BEAM_SLOPE	0.5	/* max slope of a beam */
-#define BEAM_STUB	8.0	/* length of stub for flag under beam */ 
+#define BEAM_STUB	7.0	/* length of stub for flag under beam */ 
 #define SLUR_SLOPE	1.0	/* max slope of a slur */
 #define GSTEM		15	/* grace note stem length */
 #define GSTEM_XOFF	2.3	/* x offset for grace note stem */
@@ -117,7 +117,7 @@ enum accidentals {
 /* -- macros for program internals -- */
 
 #define STRL1		256	/* string length for file names */
-#define MAXSTAFF	16	/* max staves */
+#define MAXSTAFF	32	/* max staves */
 #define BSIZE		512	/* buffer size for one input string */
 
 #define BREVE		(BASE_LEN * 2)	/* double note (square note) */
@@ -168,7 +168,7 @@ struct map {		/* voice mapping */
 extern struct map *maps; /* note mappings */
 
 struct note {		/* note head */
-	short len;		/* note duration (# pts in [1] if space) */
+	int len;		/* note duration (# pts in [1] if space) */
 	signed char pit;	/* absolute pitch from source - used for ties and map */
 	unsigned char acc;	/* code for accidental & index in micro_tb */
 	unsigned char sl1;	/* slur start */
@@ -344,7 +344,8 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 	int color;
 	float x;		/* x offset */
 	signed char y;		/* y offset of note head */
-	signed char ymn, ymx, yav; /* min, max, avg note head y offset */
+	signed char ymn, ymx;	/* min, max, note head y offset */
+	signed char mid;	// y offset of the staff middle line
 	float xmx;		/* max h-pos of a head rel to top
 				 * width when STBRK */
 	float xs, ys;		/* coord of stem end / bar height */
@@ -406,7 +407,7 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 			short wmeasure;		/* duration of a measure */
 			unsigned char nmeter;	/* number of meter elements */
 			char expdur;		/* explicit measure duration */
-#define MAX_MEASURE 6
+#define MAX_MEASURE 16
 			struct {
 				char top[8];	/* top value */
 				char bot[2];	/* bottom value */
@@ -482,14 +483,15 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 };
 
 /* parse definition */
-struct {
+struct parse {
 	struct SYMBOL *first_sym; /* first symbol */
 	struct SYMBOL *last_sym; /* last symbol */
 	int abc_vers;		/* ABC version = (H << 16) + (M << 8) + L */
 	char *deco_tb[DC_NAME_SZ]; /* decoration names */
 	unsigned short micro_tb[MAXMICRO]; /* microtone values [ (n-1) | (d-1) ] */
 	int abc_state;		/* parser state */
-} parse;
+};
+extern struct parse parse;
 
 #define	FONT_UMAX 10		/* max number of user fonts 0..9 */
 enum e_fonts {
@@ -524,7 +526,7 @@ struct FORMAT { 		/* struct for page layout */
 	float gutter, indent, infospace, slurheight, notespacingfactor, scale;
 	float staffsep, sysstaffsep, maxstaffsep, maxsysstaffsep, stretchlast;
 	int abc2pscompat, alignbars, aligncomposer, autoclef;
-	int barsperstaff, breakoneoln, bstemdown, cancelkey;
+	int barsperstaff, breakoneoln, bstemdown, cancelkey, capo;
 	int combinevoices, contbarnb, continueall, custos;
 	int dblrepbar, decoerr, dynalign, flatbeams, infoline;
 	int gchordbox, graceslurs, graceword,gracespace, hyphencont;
@@ -558,12 +560,14 @@ typedef struct SYMBOL *INFO[26]; /* information fields ('A' .. 'Z') */
 extern INFO info;
 
 extern char *outbuf;		/* output buffer.. should hold one tune */
+extern int outbufsz;		/* size of outbuf */
 extern char *mbf;		/* where to PUTx() */
 extern int use_buffer;		/* 1 if lines are being accumulated */
 
 extern int outft;		/* last font in the output file */
 extern int tunenum;		/* number of current tune */
 extern int pagenum;		/* current page number */
+extern int pagenum_nr;		/* current page (non-resettable) */
 extern int nbar;		/* current measure number */
 extern int in_page;
 extern int defl;		/* decoration flags */
@@ -675,7 +679,7 @@ struct VOICE_S {
 							/* parsing */
 	struct SYMBOL *last_note;	/* last note or rest */
 	char *map_name;
-	short ulen;			/* unit note length */
+	int ulen;			/* unit note length */
 	unsigned char microscale;	/* microtone scale */
 	unsigned char mvoice;		/* main voice when voice overlay */
 };
@@ -726,7 +730,7 @@ struct SYSTEM {			/* staff system */
 //		struct clef_s clef;
 	} voice[MAXVOICE];
 };
-struct SYSTEM *cursys;		/* current staff system */
+extern struct SYSTEM *cursys;		/* current staff system */
 
 /* -- external routines -- */
 /* abcm2ps.c */
@@ -761,7 +765,7 @@ void close_page(void);
 float get_bposy(void);
 void open_fout(void);
 void write_buffer(void);
-int (*output)(FILE *out, const char *fmt, ...)
+extern int (*output)(FILE *out, const char *fmt, ...)
 #ifdef __GNUC__
 	__attribute__ ((format (printf, 2, 3)))
 #endif
