@@ -1,7 +1,7 @@
 /*
  * Generic ABC parser.
  *
- * Copyright (C) 1998-2017 Jean-François Moine
+ * Copyright (C) 1998-2018 Jean-François Moine
  * Adapted from abc2ps, Copyright (C) 1996, 1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@
 /* global values */
 int severity;			/* error severity */
 
-static short ulen;		/* unit note length set by M: or L: */
+static int ulen;		/* unit note length set by M: or L: */
 static short meter;		/* upper value of time sig for n-plets */
 static unsigned char microscale; /* current microtone scale */
 static signed char vover;	/* voice overlay (1: single bar, -1: multi-bar */
@@ -36,8 +36,6 @@ static char g_char_tb[128];
 static char *g_deco_tb[128];		/* global decoration names */
 static unsigned short g_micro_tb[MAXMICRO]; /* global microtone values */
 
-#define VOICE_NAME_SZ 64	/* max size of a voice name */
-
 static char *abc_fn;		/* current source file name */
 static int linenum;		/* current source line number */
 static int colnum;		/* current source column number */
@@ -47,6 +45,8 @@ static struct SYMBOL *last_sym;	/* last symbol for errors */
 static short nvoice;		/* number of voices (0..n-1) */
 struct VOICE_S *curvoice;	/* current voice while parsing */
 
+struct parse parse;
+
 /* char table for note line parsing */
 #define CHAR_BAD 0
 #define CHAR_IGN 1
@@ -108,8 +108,6 @@ static char char_tb[256] = {
 
 static const char all_notes[] = "CDEFGABcdefgab";
 
-static char *parse_len(char *p,
-			int *p_len);
 static int parse_info(char *p);
 static char *parse_gchord(char *p);
 static int parse_line(char *p);
@@ -421,6 +419,8 @@ static char *parse_acc(char *p,
 			break;
 		}
 		p = parse_acc_pit(p, &pit, &acc);
+		if (acc < 0)
+			break;
 		s->u.key.pits[nacc] = pit;
 		s->u.key.accs[nacc++] = acc;
 		while (isspace((unsigned char) *p))
@@ -533,19 +533,13 @@ static void parse_clef(struct SYMBOL *s,
 			name += 4;
 		} else {
 			syntax("Unknown clef", name);
+			clef = TREBLE;
 		}
 	}
 
 	if (clef >= 0) {
-		switch (*name) {
-		case '1':
-		case '2':
-		case '3':
-		case '4':
-		case '5':
+		if (isdigit((unsigned char) *name))
 			clef_line = *name++ - '0';
-			break;
-		}
 		if (name[1] == '8') {
 			switch (*name) {
 			case '^':
@@ -570,6 +564,8 @@ static void parse_clef(struct SYMBOL *s,
 		warn = middle;
 		/* 'middle=<note pitch>' */
 		parse_acc_pit(middle, &pit, &acc);
+		if (acc < 0)			// if error
+			pit = 22;
 
 		if (clef < 0)
 			clef = line_tb[(pit + 7) % 7];
@@ -837,11 +833,9 @@ unk:
 		parse_clef(s, clef_name, clef_middle);
 	}
 	if (p_map) {
-		char name[VOICE_NAME_SZ];
-
-		strcpy(name, "%%voicemap ");
-		get_str(&name[11], p_map, VOICE_NAME_SZ - 12);
-		abc_new(ABC_T_PSCOM, name);
+		strcpy(tex_buf, "%%voicemap ");
+		get_str(&tex_buf[11], p_map, TEX_BUF_SZ - 12);
+		abc_new(ABC_T_PSCOM, tex_buf);
 	}
 }
 
@@ -853,6 +847,7 @@ static char *get_len(char *p,
 	char *error_txt = NULL;
 
 	if (strcmp(p, "auto") == 0) {		/* L:auto */
+		ulen = 15120;			// 2*2*2*2*3*3*3*5*7
 		s->u.length.base_length = -1;
 		return error_txt;
 	}
@@ -1199,6 +1194,8 @@ static char *get_user(char *p,
 		p++;
 /*fixme: 'U: <char> = "text"' is not treated */
 	get_deco(p, &s->u.user.value);
+	if (!s->u.user.value)
+		return 0;
 
 	/* treat special pseudo decorations */
 	value = parse.deco_tb[s->u.user.value - 128];
@@ -1221,7 +1218,6 @@ static char *parse_voice(char *p,
 {
 	int voice;
 	char *error_txt = NULL;
-	char name[VOICE_NAME_SZ];
 	char *clef_name, *clef_middle, *clef_stlines, *clef_scale;
 	char *p_octave, *p_cue, *p_map;
 	signed char *p_stem;
@@ -1325,14 +1321,14 @@ static struct kw_s {
 		p += kw->len;
 		switch (kw->index) {
 		case 0:			/* name */
-			p = get_str(name, p, VOICE_NAME_SZ);
-			s->u.voice.fname = getarena(strlen(name) + 1);
-			strcpy(s->u.voice.fname, name);
+			p = get_str(tex_buf, p, TEX_BUF_SZ);
+			s->u.voice.fname = getarena(strlen(tex_buf) + 1);
+			strcpy(s->u.voice.fname, tex_buf);
 			break;
 		case 1:			/* subname */
-			p = get_str(name, p, VOICE_NAME_SZ);
-			s->u.voice.nname = getarena(strlen(name) + 1);
-			strcpy(s->u.voice.nname, name);
+			p = get_str(tex_buf, p, TEX_BUF_SZ);
+			s->u.voice.nname = getarena(strlen(tex_buf) + 1);
+			strcpy(s->u.voice.nname, tex_buf);
 			break;
 		case 2:			/* merge */
 			s->u.voice.merge = 1;
@@ -1401,9 +1397,9 @@ static struct kw_s {
 		parse_clef(s, clef_name, clef_middle);
 	}
 	if (p_map) {
-		strcpy(name, "%%voicemap ");
-		get_str(&name[11], p_map, VOICE_NAME_SZ - 12);
-		abc_new(ABC_T_PSCOM, name);
+		strcpy(tex_buf, "%%voicemap ");
+		get_str(&tex_buf[11], p_map, TEX_BUF_SZ - 12);
+		abc_new(ABC_T_PSCOM, tex_buf);
 	}
 	return error_txt;
 }
@@ -1630,24 +1626,6 @@ char *parse_acc_pit(char *p,
 	return p;
 }
 
-/* -- parse note or rest with pitch and length -- */
-/* in case of error, 'accidental' is set to -1 */
-static char *parse_basic_note(char *p,
-				int *pitch,
-				int *length,
-				int *accidental,
-				int *stemless)
-{
-	p = parse_acc_pit(p, pitch, accidental);
-	if (*p == '0') {
-		*stemless = 1;
-		p++;
-	} else {
-		*stemless = 0;
-	}
-	return parse_len(p, length);
-}
-
 /* -- parse the decorations of notes and bars -- */
 static char *parse_deco(char *p,
 			 struct decos *deco,
@@ -1810,12 +1788,13 @@ static char *parse_gchord(char *p)
 
 /* -- parse a note length -- */
 static char *parse_len(char *p,
+			int dur_u,
 			int *p_len)
 {
 	int len, fac;
 	char *q;
 
-	len = BASE_LEN;
+	len = dur_u;
 	if (isdigit((unsigned char) *p)) {
 		len *= strtol(p, &q, 10);
 		if (len <= 0) {
@@ -1851,6 +1830,7 @@ static int parse_line(char *p)
 {
 	struct SYMBOL *s;
 	char *q, c;
+	char *dot = NULL;
 	struct SYMBOL *last_note_sav = NULL;
 	struct decos dc_sav;
 	int i, flags, flags_sav = 0, slur;
@@ -2028,8 +2008,10 @@ static int parse_line(char *p)
 			/* fall thru */
 		case CHAR_DECO:
 			if (p[-1] == '.') {
-				if (*p == '(' || *p == '-')
+				if (*p == '(' || *p == '-') {
+					dot = p;
 					break;
+				}
 //				if (*p == '|') {
 //					p = parse_bar(p + 1);
 //					parse.last_sym->u.bar.dotted = 1;
@@ -2147,6 +2129,10 @@ static int parse_line(char *p)
 					syntax("Invalid 'r' in tuplet", p);
 					break;
 				}
+				if (pplet >= 128 || qplet >= 128 || rplet >= 128) {
+					syntax("Invalid 'p:q:r' in tuplet", p);
+					break;
+				}
 				if (qplet == 0)
 					qplet = meter % 3 == 0 ? 3 : 2;
 				s = abc_new(ABC_T_TUPLET, NULL);
@@ -2171,7 +2157,7 @@ static int parse_line(char *p)
 				break;
 			}
 			slur <<= 4;
-			if (p[-2] == '.' && dc.n == 0)
+			if (p == dot + 1 && dc.n == 0)
 				slur |= SL_DOTTED;
 			switch (*p) {
 			case '\'':
@@ -2231,7 +2217,7 @@ static int parse_line(char *p)
 			if (!curvoice->last_note
 			 || curvoice->last_note->abc_type != ABC_T_NOTE)
 				goto bad_char;
-			if (p[-2] == '.' && dc.n == 0)
+			if (p == dot + 1 && dc.n == 0)
 				tie_pos = SL_DOTTED;
 			else
 				tie_pos = 0;
@@ -2370,8 +2356,8 @@ static char *parse_note(char *p,
 		/* fall thru */
 	case 'z':
 		s->abc_type = ABC_T_REST;
-		p = parse_len(p + 1, &len);
-		s->u.note.notes[0].len = len * ulen / BASE_LEN;
+		p = parse_len(p + 1, ulen, &len);
+		s->u.note.notes[0].len = len;
 		goto do_brhythm;
 	case '[':			/* '[..]' = chord */
 		chord = 1;
@@ -2415,18 +2401,20 @@ static char *parse_note(char *p,
 			}
 		}
 		p = parse_deco(p, &dc, m);	/* note head decorations */
-		p = parse_basic_note(p, &pit, &len, &acc, &n);
-		if (!(flags & ABC_F_GRACE))
-			len = len * ulen / BASE_LEN;
-		else
-			len /= 8;		/* for grace note alone */
-
+		p = parse_acc_pit(p, &pit, &acc);
+		if (*p == '0') {
+			nostem = 1;
+			p++;
+		}
+		p = parse_len(p, (flags & ABC_F_GRACE) ?
+					BASE_LEN / 8 :	// for grace note alone
+					ulen,
+				&len);
 		s->u.note.notes[m].pit = pit;
 		s->pits[m] = pit;
 		s->u.note.notes[m].len = len;
 		s->u.note.notes[m].acc = acc;
 		s->u.note.notes[m].color = -1;
-		nostem |= n;
 
 		if (chord) {
 			for (;;) {
@@ -2465,14 +2453,14 @@ static char *parse_note(char *p,
 		if (*p == ']') {
 			p++;
 			if (*p == '0') {
-				nostem |= 1;
+				nostem = 1;
 				p++;
 			}
 			if (*p == '/' || isdigit((unsigned char) *p)) {
-				p = parse_len(p, &len);
+				p = parse_len(p, ulen, &len);
 				for (j = 0; j < m; j++) {
-					n = len * s->u.note.notes[j].len;
-					s->u.note.notes[j].len = n / BASE_LEN;
+					s->u.note.notes[j].len =
+						len * s->u.note.notes[j].len / ulen;
 				}
 			}
 			break;
