@@ -36,7 +36,7 @@ static float remy;		/* remaining vertical space in page */
 static float bposy;		/* current position in buffered data */
 static int nepsf;		/* counter for -E/-g output files */
 static int nbpages;		/* number of pages in the output file */
-static int outbufsz;		/* size of outbuf */
+	int outbufsz;		/* size of outbuf */
 static char outfnam[FILENAME_MAX]; /* internal file name for open/close */
 static struct FORMAT *p_fmt;	/* current format while treating a new page */
 
@@ -46,6 +46,11 @@ int in_page;			/* filling a PostScript page */
 char *outbuf;			/* output buffer.. should hold one tune */
 char *mbf;			/* where to a2b() */
 int use_buffer;			/* 1 if lines are being accumulated */
+int (*output)(FILE *out, const char *fmt, ...)
+#ifdef __GNUC__
+	__attribute__ ((format (printf, 2, 3)))
+#endif
+	;
 
 /* -- cut off extension on a file identifier -- */
 static void cutext(char *fid)
@@ -210,11 +215,17 @@ static void init_ps(char *str)
 	output = fprintf;
 	user_ps_write();
 	define_fonts();
-	if (!epsf)
+	if (!epsf) {
 		fprintf(fout, "/setpagedevice where{pop\n"
-			"	<</PageSize[%.0f %.0f]>>setpagedevice}if\n",
+			"	<</PageSize[%.0f %.0f]",
 				p_fmt->pagewidth * PPI_96_72,
 				p_fmt->pageheight * PPI_96_72);
+		if (cfmt.gutter)
+			fprintf(fout,
+			 "\n	/BeginPage{1 and 0 eq{%.1f 0 T}{-%.1f 0 T}ifelse}bind\n	",
+				cfmt.gutter, cfmt.gutter);
+		fprintf(fout, ">>setpagedevice}if\n");
+	}
 	fprintf(fout, "%%%%EndSetup\n");
 	file_initialized = 1;
 }
@@ -246,11 +257,10 @@ static void close_fout(void)
 		goto out1;
 	m = ftell(fout);
 	if (epsf || svg == 1)
-		fprintf(stderr, "Output written on %s (%ld bytes)\n",
+		printf("Output written on %s (%ld bytes)\n",
 			outfnam, m);
 	else
-		fprintf(stderr,
-			"Output written on %s (%d page%s, %d title%s, %ld bytes)\n",
+		printf("Output written on %s (%d page%s, %d title%s, %ld bytes)\n",
 			outfnam,
 			nbpages, nbpages == 1 ? "" : "s",
 			tunenum, tunenum == 1 ? "" : "s",
@@ -386,6 +396,18 @@ static void format_hf(char *d, char *p)
 			}
 			d += sprintf(d, "%d", pagenum);
 			break;
+		case 'Q':		/* non-resetting page number */
+			if (p[1] == '0') {
+				p++;
+				if (pagenum_nr & 1)
+					break;
+			} else if (p[1] == '1') {
+				p++;
+				if ((pagenum_nr & 1) == 0)
+					break;
+			}
+			d += sprintf(d, "%d", pagenum_nr);
+			break;
 		case 'T':		/* tune title */
 			q = &info['T' - 'A']->text[2];
 			tex_str(q);
@@ -574,9 +596,6 @@ static void init_page(void)
 			"%% --- width %.1f\n",		/* for index */
 			(pwidth - cfmt.leftmargin - cfmt.rightmargin) /
 					cfmt.scale);
-	if (cfmt.gutter != 0)
-		output(fout, "%.1f 0 T\n", (pagenum & 1) ?
-					cfmt.gutter : -cfmt.gutter);
 
 	remy = maxy = pheight - cfmt.topmargin - cfmt.botmargin;
 
@@ -605,6 +624,7 @@ static void init_page(void)
 	if (cfmt.footer)
 		remy -= headfooter(0, pwidth, pheight);
 	pagenum++;
+	pagenum_nr++;
 	outft = -1;
 }
 
@@ -884,7 +904,8 @@ void write_buffer(void)
 	outft = outft_sav;
 	bposy = 0;
 	ln_num = 0;
-	use_buffer = 0;
+	if (epsf != 3)
+		use_buffer = 0;
 }
 
 /* -- add a block of commmon margins / scale in the output buffer -- */
