@@ -54,6 +54,7 @@ static struct format {
 	{"breakoneoln", &cfmt.breakoneoln, FORMAT_B, 0},
 	{"bstemdown", &cfmt.bstemdown, FORMAT_B, 0},
 	{"cancelkey", &cfmt.cancelkey, FORMAT_B, 0},
+	{"capo", &cfmt.capo, FORMAT_I, 0},
 	{"combinevoices", &cfmt.combinevoices, FORMAT_I, 0},
 	{"composerfont", &cfmt.font_tb[COMPOSERFONT], FORMAT_F, 0},
 	{"composerspace", &cfmt.composerspace, FORMAT_U, 0},
@@ -140,7 +141,7 @@ static struct format {
 	{"titleformat", &cfmt.titleformat, FORMAT_S, 0},
 	{"titleleft", &cfmt.titleleft, FORMAT_B, 0},
 	{"titlespace", &cfmt.titlespace, FORMAT_U, 0},
-	{"titletrim", &cfmt.titletrim, FORMAT_B, 0},
+	{"titletrim", &cfmt.titletrim, FORMAT_I, 0},
 	{"timewarn", &cfmt.timewarn, FORMAT_B, 0},
 	{"topmargin", &cfmt.topmargin, FORMAT_U, 1},
 	{"topspace", &cfmt.topspace, FORMAT_U, 0},
@@ -399,7 +400,7 @@ void set_format(void)
 	if (!svg && epsf <= 1)
 		f->pango = 1;
 	else
-		lock_fmt(&cfmt.pango);	/* SVG output does not use panga */
+		lock_fmt(&cfmt.pango);	/* SVG output does not use pango */
 #endif
 	f->rbdbstop = 1;
 	f->rbmax = 4;
@@ -420,9 +421,9 @@ void set_format(void)
 	if (svg || epsf > 2) {		// SVG output
 		fontspec(&f->font_tb[ANNOTATIONFONT], sans, 0, 12.0);
 		fontspec(&f->font_tb[COMPOSERFONT], serif_italic, 0, 14.0);
-		fontspec(&f->font_tb[FOOTERFONT], serif, 0, 16.0); /* not scaled */
+		fontspec(&f->font_tb[FOOTERFONT], serif, 0, 16.0);
 		fontspec(&f->font_tb[GCHORDFONT], sans, 0, 12.0);
-		fontspec(&f->font_tb[HEADERFONT], serif, 0, 16.0); /* not scaled */
+		fontspec(&f->font_tb[HEADERFONT], serif, 0, 16.0);
 		fontspec(&f->font_tb[HISTORYFONT], serif, 0, 16.0);
 		fontspec(&f->font_tb[INFOFONT],	serif_italic, 0, 14.0); /* same as composer by default */
 		fontspec(&f->font_tb[MEASUREFONT], serif_italic, 0, 14.0);
@@ -438,9 +439,9 @@ void set_format(void)
 	} else {			// PS output
 		fontspec(&f->font_tb[ANNOTATIONFONT], helvetica, 0, 12.0);
 		fontspec(&f->font_tb[COMPOSERFONT], times_italic, 0, 14.0);
-		fontspec(&f->font_tb[FOOTERFONT], times, 0, 16.0); /* not scaled */
+		fontspec(&f->font_tb[FOOTERFONT], times, 0, 16.0);
 		fontspec(&f->font_tb[GCHORDFONT], helvetica, 0, 12.0);
-		fontspec(&f->font_tb[HEADERFONT], times, 0, 16.0); /* not scaled */
+		fontspec(&f->font_tb[HEADERFONT], times, 0, 16.0);
 		fontspec(&f->font_tb[HISTORYFONT], times, 0, 16.0);
 		fontspec(&f->font_tb[INFOFONT],	times_italic, 0, 14.0); /* same as composer by default */
 		fontspec(&f->font_tb[MEASUREFONT], times_italic, 0, 14.0);
@@ -607,15 +608,19 @@ float scan_u(char *str, int type)
 	int nch;
 
 	if (sscanf(str, "%f%n", &a, &nch) == 1) {
-		if (str[nch] == '\0' || str[nch] == ' '
-		 || !strncasecmp(str + nch, "pt", 2)) {
+		str += nch;
+		if (a == 0)
+			return 0;
+		if (*str == '\0' || *str == ' ') {
 			if (type != 0)
-				error(0, NULL, "Bad unit \"%s\"", str);
+				error(0, NULL, "No unit \"%s\"", str - nch);
 			return a PT;
 		}
-		if (!strncasecmp(str + nch, "cm", 2))
+		if (!strncasecmp(str, "pt", 2))
+			return a PT;
+		if (!strncasecmp(str, "cm", 2))
 			return type ? a CM : a * 28.35;
-		if (!strncasecmp(str + nch, "in", 2))
+		if (!strncasecmp(str, "in", 2))
 			return type ? a IN : a * 72;
 	}
 	error(1, NULL, "Unknown unit value \"%s\"", str);
@@ -635,7 +640,8 @@ static int get_posit(char *p)
 	 || strcmp(p, "above") == 0)
 		return SL_ABOVE;
 	if (strcmp(p, "down") == 0
-	 || strcmp(p, "below") == 0)
+	 || strcmp(p, "below") == 0
+	 || strcmp(p, "under") == 0)
 		return SL_BELOW;
 	if (strcmp(p, "hidden") == 0
 	 || strcmp(p, "opposite") == 0)
@@ -752,7 +758,7 @@ static void g_fspc(char *p,
 			fsize = v;
 	}
 	fontspec(f,
-		 strcmp(fname, "*") != 0 ? fname : 0,
+		 strcmp(fname, "*") != 0 ? fname : NULL,
 		 encoding,
 		 fsize);
 	if (file_initialized <= 0)
@@ -1183,6 +1189,8 @@ void interpret_fmt_line(char *w,		/* keyword */
 		switch (fd->subtype) {
 #ifdef HAVE_PANGO
 		case 2:				/* %%pango = 0, 1 or 2 */
+			if (svg || epsf > 1)	// if SVG output
+				break;
 			if (*p == '2') {
 				cfmt.pango = 2;
 				break;
@@ -1352,14 +1360,7 @@ void interpret_fmt_line(char *w,		/* keyword */
 		else
 			strcpy(*((char **) fd->v), p);
 		if (fd->subtype == 1) {			// musicfont
-			if ((strncmp(p, "url(", 4)
-			  && strncmp(p, "local(", 4))
-			 || p[strlen(p) - 1] != ')') {
-				cfmt.musicfont = NULL;
-				error(1, NULL,
-					"%s can be only url(..) or local(..)", w);
-				return;
-			}
+			// (no control)
 			svg_font_switch();
 		}
 		break;
@@ -1423,5 +1424,5 @@ void set_font(int ft)
 /* -- get the encoding of a font -- */
 int get_font_encoding(int ft)
 {
-	return font_enc[ft];
+	return font_enc[cfmt.font_tb[ft].fnum];
 }
