@@ -619,12 +619,12 @@ void frontend(unsigned char *s,
 			goto next_eol;
 		}
 		if (histo) {			/* H: continuation */
-			if ((s[1] == ':'
-			  && (isalpha(*s) || *s == '+'))
+			if ((s[1] == ':' && isalpha(*s))
 			 || (*s == '%' && strchr(prefix, s[1]))) {
 				histo = 0;
 			} else {
-				txt_add((unsigned char *) "H:", 2);
+				if (*s != '+' || s[1] != ':')
+					txt_add((unsigned char *) "+:", 2);
 				goto next;
 			}
 		}
@@ -658,6 +658,8 @@ void frontend(unsigned char *s,
 				s++;
 				l--;
 			}
+			if (l <= 0)
+				goto ignore;
 			txt_add((unsigned char *) "%%", 2);
 			goto pcinfo;
 		}
@@ -699,6 +701,8 @@ pscom:
 				s++;
 				l--;
 			}
+			if (l <= 0)
+				goto ignore;
 			txt_add((unsigned char *) "%%", 2);
 			if (strncmp((char *) s, "begin", 5) == 0) {
 				q = begin_end = s + 5;
