@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2017 Jean-François Moine
+ * Copyright (C) 1998-2018 Jean-François Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -128,7 +128,7 @@ static void cap_str(char *p)
 /* -- return the character width -- */
 float cwid(unsigned char c)
 {
-	if (c > 0x80) {
+	if (c >= 0x80) {
 		if (c < 0xc0)
 			return 0;	// not start of utf8 character
 		c = 'a';
@@ -325,7 +325,6 @@ void pg_init(void)
 		cfmt.pango = 0;
 	} else {
 		pango_layout_set_wrap(layout, PANGO_WRAP_WORD);
-//		pango_layout_set_spacing(layout, 0);
 		pg_str = g_string_sized_new(256);
 	}
 }
@@ -336,16 +335,55 @@ void pg_reset_font(void)
 
 static void desc_font(int fnum)
 {
-	char font_name[128], *p;
+	int i;
+	char font_name[128], *p, *q;
+static char *bad_tb[] = {
+	"Times-Roman", "Times", "Helvetica", "Courier"
+};
+static char *good_tb[] = {
+	"serif", "serif", "sans-serif", "monospace"
+};
 
-	if (desc_tb[fnum] == 0) {
+	// build a font description replacing X11 font names by generic names
+	if (!desc_tb[fnum]) {
 		p = font_name;
-		sprintf(p, "%s 10", fontnames[fnum]);
+		q = fontnames[fnum];
+		for (i = 0; i < sizeof bad_tb / sizeof bad_tb[0]; i++) {
+			if (strncmp(q, bad_tb[i], strlen(bad_tb[i])) == 0) {
+				p += sprintf(p, "%s", good_tb[i]);
+				q += strlen(bad_tb[i]);
+				break;
+			}
+		}
+		snprintf(p, sizeof font_name - 5, "%s 10", q);	// (dummy size)
+
+		// change '-' to a space
 		while (*p != '\0') {
 			if (*p == '-')
 				*p = ' ';
+			if (isupper(*p))
+				*p = tolower(*p);
 			p++;
 		}
+
+		// add spaces between styles/weight
+		p = strstr(&font_name[1], "italic");
+		if (p && p[-1] != ' ') {
+			memmove(p + 1, p, strlen(p) + 1);
+			*p = ' ';
+		}
+		p = strstr(&font_name[1], "oblique");
+		if (p && p[-1] != ' ') {
+			memmove(p + 1, p, strlen(p) + 1);
+			*p = ' ';
+		}
+		p = strstr(&font_name[1], "bold");
+		if (p && p[-1] != ' ') {
+			memmove(p + 1, p, strlen(p) + 1);
+			*p = ' ';
+		}
+
+		// create the font description
 		desc_tb[fnum] = pango_font_description_from_string(font_name);
 	}
 }
@@ -369,15 +407,10 @@ static void pg_line_output(PangoLayoutLine *line)
 		PangoFont *font = analysis->font;
 		PangoFcFont *fc_font = PANGO_FC_FONT(font);
 		FT_Face face = pango_fc_font_lock_face(fc_font);
-		PangoFontDescription *ftdesc =
-				pango_font_describe(font);
+		PangoFontDescription *ftdesc = pango_font_describe(font);
 		int wi = pango_font_description_get_size(ftdesc);
 		int i, c;
 
-		if (pango_font_description_get_size(ftdesc) != wi) {
-			wi = pango_font_description_get_size(ftdesc);
-			fontname = NULL;
-		}
 		for (i = 0; i < glyphs->num_glyphs; i++) {
 			glyph_info = &glyphs->glyphs[i];
 			c = glyph_info->glyph;
@@ -399,18 +432,17 @@ static void pg_line_output(PangoLayoutLine *line)
 					fontname = FT_Get_Postscript_Name(face);
 					if (glypharray)
 						a2b("]glypharray");
-						a2b("\n");
-					a2b("/%s %.1f selectfont[",
+					a2b("\n/%s %.1f selectfont[",
 						fontname,
 						(float) wi / PG_SCALE);
 					glypharray = 1;
 				}
-				FT_Get_Glyph_Name((FT_FaceRec *) face, c,
+				FT_Get_Glyph_Name(face, c,
 						tmp, sizeof tmp);
 				a2b("/%s", tmp);
 			} else {
-				a2b("%% glyph: %s %d\n",
-					FT_Get_Postscript_Name(face), c);
+				error(0, NULL, "!! no glyph %d in %s-%s\n",
+					c, face->family_name, face->style_name);
 			}
 		}
 		pango_fc_font_unlock_face(fc_font);
@@ -434,7 +466,7 @@ static void str_font_change(int start,
 		f->size = 8;
 	}
 	desc_font(fnum);
-	
+
 	attr1 = pango_attr_font_desc_new(desc_tb[fnum]);
 	attr1->start_index = start;
 	attr1->end_index = end;
@@ -494,8 +526,6 @@ static void str_pg_out(char *p, int action)
 	int wi;
 	float w;
 
-//fixme: test
-//a2b("\n%% t: '%s'\n", p);
 	if (out_pg_ft != curft)
 		out_pg_ft = -1;
 
@@ -591,7 +621,6 @@ static void pg_para_output(int job)
 				wi = pango_font_description_get_size(ftdesc);
 				fontname = NULL;
 			}
-//printf("font size: %.2f\n", (float) wi / PG_SCALE);
 
 			pango_layout_index_to_pos(layout, item->offset, &pos);
 			x = pos.x;
@@ -692,7 +721,7 @@ static void pg_write_text(char *s, int job, float parskip)
 			continue;
 		}
 //fixme: maybe not useful
-		p [-1] = ' ';
+		p[-1] = ' ';
 	}
 	tex_str(s);
 	str_set_font(tex_buf);
@@ -1211,9 +1240,10 @@ static int put_wline(char *p,
 		}
 		p += 2;
 	}
-	r = 0;
+	r = NULL;
 	q = p;
-	if (isdigit((unsigned char) *p) || p[1] == '.') {
+	if (isdigit((unsigned char) *p)
+	 || (*p != '\0' && p[1] == '.')) {
 		while (*p != '\0') {
 			p++;
 			if (*p == ' '
@@ -1226,7 +1256,7 @@ static int put_wline(char *p,
 			p++;
 	}
 
-	if (r != 0) {
+	if (r) {
 		sep = *r;
 		*r = '\0';
 		a2b("%.1f 0 M", x);
@@ -1361,8 +1391,12 @@ void put_history(void)
 			put_inf(s2);
 			if ((s2 = s2->next) == NULL)
 				break;
-			bskip(h);
-			a2b("%.2f 0 M ", w);
+			if (s2->text[0] == '+' && s2->text[1] == ':') {
+				put_str(" ", A_LEFT);
+			} else {
+				bskip(h);
+				a2b("%.2f 0 M ", w);
+			}
 		}
 		bskip(h * 1.2);
 		buffer_eob(0);
@@ -1379,10 +1413,16 @@ static char buf[STRL1];
 	if (cfmt.titletrim) {
 		q = strrchr(p, ',');
 		if (q) {
-			if (q[1] != ' ' || !isupper((unsigned char) q[2])
-			 || strlen(q) > 7	/* word no more than 5 characters */
-			 || strchr(q + 2, ' '))
+			if (q[1] != ' ' || !isupper((unsigned char) q[2])) {
 				q = NULL;
+			} else if (cfmt.titletrim == 1) {	// (true)
+				if (strlen(q) > 7	/* word no more than 5 characters */
+				 || strchr(q + 2, ' '))
+					q = NULL;
+			} else {
+				if (strlen(q) > cfmt.titletrim - 2)
+					q = NULL;
+			}
 		}
 	}
 	if (title != info['T' - 'A']
