@@ -386,7 +386,7 @@ static char ps_head[] =
 	/* x y xxsig - old time signatures ('o', 'o.', 'c' 'c.') */
 	"/pmsig{0.3 SLW 12 add M currentpoint 5 0 360 arc stroke}!\n"
 	"/pMsig{2 copy pmsig 12 add M currentpoint 1.3 0 360 arc fill}!\n"
-	"/imsig{0.3 SLW 12 add 2 copy 5 add M 5 90 270 arc stroke}!\n"
+	"/imsig{0.3 SLW 12 add 2 copy 5 add M 5 60 300 arc stroke}!\n"
 	"/iMsig{2 copy imsig 12 add M currentpoint 1.3 0 360 arc fill}!\n"
 
 	/* (top) (bot) x y tsig - time signature */
@@ -870,14 +870,19 @@ void define_symbols(void)
 
 	// if a music font, give it a name
 	if (p) {
-		r = strrchr(p, '.');	// remove the file type
-		if (!r)
-			r = p + strlen(p) - 1;		// ')'
-		q = strrchr(p, DIRSEP);
-		if (!q)
-			q = strchr(p, '(');
-		q++;
-		fprintf(fout, "/music/%.*s def\n", (int) (r - q), q);
+		q = strchr(p, '(');
+		if (q) {		// hope "url(...)"
+			q++;
+			r = strrchr(q, '.');	// remove the file type
+			if (!r)
+				r = q + strlen(p) - 1;		// ')'
+			p = strrchr(q, DIRSEP);	// and the directory path
+			if (p)
+				q = p + 1;
+			fprintf(fout, "/music/%.*s def\n", (int) (r - q), q);
+		} else {
+			fprintf(fout, "/music/%s def\n", p);
+		}
 	}
 
 	/* len su - up stem */
